/*
    This file is part of the 'yarec' package.
    Copyright (C) 1998  Ralf W. Stephan <stephan@tmt.de>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/* This is the facade for the sox functions */
/* Supported formats: wav and aiff */

#include "file.h"
#if	defined(unix) || defined(AMIGA) || defined(__OS2__) || defined(ARM)
#include <sys/types.h>
#include <sys/stat.h>
#endif

#if defined HAVE_STRING_H || defined _LIBC
# include <string.h>
#else
# include <strings.h>
#endif

#include "st.h"
#include "util/util.h"

#define BUFSIZE 64000

extern int wavstartread (ft_t);
extern int wavread (ft_t, u32*, long);
extern int wavstartwrite (ft_t);
extern int wavwrite (ft_t, u32*, long);
extern int wavstopwrite (ft_t);
extern int aiffstartread (ft_t);
extern int aiffread (ft_t, u32*, long);
extern int aiffstartwrite (ft_t);
extern int aiffwrite (ft_t, u32*, long);
extern int aiffstopwrite (ft_t);

static void init_ft();

static struct soundstream ft_;
static int (*rheader_func_)(ft_t);
static int (*rdata_func_)(ft_t, u32*, long);
static int (*wheader_func_)(ft_t);
static int (*wdata_func_)(ft_t, u32*, long);
static int (*wstop_func_)(ft_t);
static const struct File* file_;
static int r_, b_, c_, header_read_, header_writ_;

void Format_init (const struct File* f, int contd)
{
  const char* format = Config_sget ("FILE_FORMAT");
  file_ = f;
  ft_.fp        = file_->fp;
  if (contd) return;
  
  header_read_ = header_writ_ = 0;
  init_ft();       /* This is ugly */
  
  if (!strcmp ((char*)format, "wav")) {
  	rheader_func_ = wavstartread;
  	rdata_func_ = wavread;
  	wheader_func_ = wavstartwrite;
  	wdata_func_ = wavwrite;
  	wstop_func_ = wavstopwrite;
  } else 
  if (!strcmp ((char*)format, "aiff")) {
  	rheader_func_ = aiffstartread;
  	rdata_func_ = aiffread;
  	wheader_func_ = aiffstartwrite;
  	wdata_func_ = aiffwrite;
  	wstop_func_ = aiffstopwrite;
  }
  else
  	Exception_fatal ("Riot in file/format.c!");
}

void Format_get_params (int *r, int *b, int *c)
{
  if (!header_read_) {
  	File_reset (file_);
  	(*rheader_func_)(&ft_);
  	r_ = ft_.info.rate;
  	b_ = 8*ft_.info.size;
  	c_ = ft_.info.channels;
  	header_read_ = 1;
  }
  *r = r_;
  *b = b_;
  *c = c_;
}

void Format_set_params (int r, int b, int c, int contd)
{
  if (!File_wflag (file_) || header_writ_) return;
  
  ft_.info.rate = r_ = r;
  b_ = b;
  ft_.info.size = b/8;
  ft_.info.channels = c_ = c;
  if (!contd) {
  	ft_.info.style = -1;
  	(*wheader_func_)(&ft_);
  }
  header_writ_ = 1;
}

int Format_get_chunk (u32* p, long chunksize)
{
  if (!header_read_)
  	Format_get_params (&r_, &b_, &c_);
    	
  return (*rdata_func_)(&ft_, p, chunksize);
}

void Format_write_chunk (u32* buf, long size)
{
  if (!header_writ_)
  	Exception_fatal ("format.c: write_chunk before set_params!");
  
  (*wdata_func_)(&ft_, buf, size);
}

void Format_close()
{
  if (File_wflag (file_))
  	(*wstop_func_)(&ft_);
  File_reset (file_);
}

/*-------------------------------------------------------------------*/
/* TODO: Converting to and from 32bit might be bad, performancewise. */

static void convf_32to8le (u32*, long);
static void convf_32to16le (u32*, long);
static void convf_ident (u32*, long);

conv_func_t Format_get_conversion()
{
  switch (b_) {
  case 8:  return convf_32to8le;
  case 16: return convf_32to16le;
  case 32: return convf_ident;
  default: Exception_fatal ("Riot in format.c!");
  }
}

static void convf_32to8le (u32* p, long len)
{
  u8* p8 = (u8*)p;
  long i;
  for (i=0; i<len; i++)
  	*p8++ = (*p++ >> 24)^0x80;
}

static void convf_32to16le (u32* p, long len)
{
  u16* p16 = (u16*)p;
  long i;
  for (i=0; i<len; i++) 
  	*p16++ = ((u16)(*p++ >> 16));
}

static void convf_ident (u32* p, long len) {}

/*----------------------------------------------------------*/

#if	defined(unix) || defined(AMIGA) || defined(ARM)
static int filetype(fd)
int fd;
{
	struct stat st;

	fstat(fd, &st);

	return st.st_mode & S_IFMT;
}
#endif

/* This initializes the structure used by the sox functions */
static void init_ft()
{
  ft_.info.rate      = 0;
  ft_.info.size      = -1;
  ft_.info.style     = -1;
  ft_.info.channels  = -1;
  ft_.comment   = file_->name;
  ft_.swap      = 0;
  ft_.filetype  = "auto";
  ft_.fp        = file_->fp;
  ft_.filename  = file_->name;
  ft_.h         = 0;

#if	defined(unix) || defined(AMIGA) || defined(ARM)
	ft_.seekable  = (filetype(fileno(ft_.fp)) == S_IFREG);
#else
#if	defined(DOS) || defined(__OS2__)
	ft_.seekable  = 1;
#else
	ft_.seekable  = 0;
#endif
#endif

}